////////////////////////////////////////////////////////////////////////////////////
//stelephony.h - internal definitions of stelephony.dll
////////////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#if defined(__WINDOWS__)
#include <atlbase.h>	//for IConnectionPoint
#include <objbase.h>
#endif


#include <StelephonyApi.h>
#include "Q931EventsDecoder.h" 
#include "Sink.h"

#if defined(__WINDOWS__)
#if !defined(__FUNCTION__)
const char *__FUNCTION__ = "Function";
#endif
#endif

#define STEL_FUNC()			if(0)printf("%s(): Line: %d\n", __FUNCTION__, __LINE__)
#define STEL_FUNC_START()	if(0)printf("%s(): Start\n", __FUNCTION__)
#define STEL_FUNC_END()		if(0)printf("%s(): End\n", __FUNCTION__)

#if defined (__LINUX__)
#include "PToneDecoder.h"
#include "PToneEncoder.h"
#endif

// This class will provide telepony services for a SINGLE thread
class CStelephony 
{
	CRITICAL_SECTION	m_CriticalSection;

#if defined(__WINDOWS__)
	IConnectionPoint	*pCP;		//these are declared as a member
	CSink				*pSink;
	DWORD		        dwAdvise;
	CComPtr<IPhoneToneDecoder> pToneDecoder;
#else
	PhoneToneDecoder 	*pToneDecoder;
	PhoneToneEncoder 	*pToneEncoder;
#endif
	Q931EventsDecoder	Q931EventsDecoderObj;

	stelephony_status_t InitDecoderLib();
	stelephony_status_t CleanupDecoderLib();

	stelephony_status_t InitEncoderLib();
	stelephony_status_t CleanupEncoderLib();

	stelephony_callback_functions_t callback_functions;
	void *callback_context;

	stelephony_option_t m_ToneDecoderCodec;
	stelephony_option_t m_ToneEncoderCodec;

	stelephony_status_t ToneDecoderEventControl(stelephony_event_t Event, stelephony_control_code_t ControlCode, void *optionalData);
	stelephony_status_t ToneEncoderEventControl(stelephony_event_t Event, stelephony_control_code_t ControlCode, void *optionalData);

public:
	CStelephony();
	~CStelephony();

	void EnterStelCriticalSection();
	void LeaveStelCriticalSection();

	stelephony_status_t Init();
	stelephony_status_t Cleanup();

	void SetCallbackFunctions(IN stelephony_callback_functions_t *cbf);
	void GetCallbackFunctions(OUT stelephony_callback_functions_t *cbf);
	void SetCallbackContext(void *cbc);
	void *GetCallbackContext(void);

	int DecoderLibStreamInput(void *data, int dataLength);

	stelephony_status_t EncoderLibGenerateFSKCallerID(stelephony_caller_id_t *cid_info);

	stelephony_status_t EncoderLibGenerateSwDTMF(char dtmfChar);

	stelephony_status_t EventControl(stelephony_event_t Event, stelephony_control_code_t ControlCode, void *optionalData);
};
